==> unity-ads-4.13.0-sources.jar: com/unity3d/ads/core/domain/LegacyLoadUseCase.kt <==
package com.unity3d.ads.core.domain

import android.content.Context
import com.google.protobuf.ByteString
import com.unity3d.ads.IUnityAdsLoadListener
import com.unity3d.ads.UnityAds.UnityAdsLoadError
import com.unity3d.ads.UnityAdsLoadOptions
import com.unity3d.ads.core.data.model.AdObject
import com.unity3d.ads.core.data.model.LoadResult
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_AD_MARKUP_PARSING
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_NOT_INITIALIZED
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_OPPORTUNITY_ID
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_OPPORTUNITY_ID_USED
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_PLACEMENT_NULL
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_TIMEOUT
import com.unity3d.ads.core.data.model.InitializationState.INITIALIZED
import com.unity3d.ads.core.data.model.InitializationState.NOT_INITIALIZED
import com.unity3d.ads.core.data.model.InitializationState.FAILED
import com.unity3d.ads.core.data.model.InitializationState.INITIALIZING
import com.unity3d.ads.core.data.model.OperationType
import com.unity3d.ads.core.data.repository.AdRepository
import com.unity3d.ads.core.data.repository.SessionRepository
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.OPERATION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_AD_MARKUP_PARSE
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_DEBUG
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_NOT_INITIALIZED
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_OPPORTUNITY_ID
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_OPPORTUNITY_USED
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_PLACEMENT_NULL
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_TIMEOUT
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_TIMEOUT_INITIALIZATION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_UNCAUGHT_EXCEPTION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.STATE
import com.unity3d.ads.core.extensions.*
import com.unity3d.ads.core.extensions.toByteString
import com.unity3d.services.UnityAdsConstants.Messages.MSG_INTERNAL_ERROR
import com.unity3d.services.banners.UnityBannerSize
import com.unity3d.services.core.log.DeviceLog
import gatewayprotocol.v1.DiagnosticEventRequestOuterClass.DiagnosticAdType
import gatewayprotocol.v1.DiagnosticEventRequestOuterClass.DiagnosticAdType.DIAGNOSTIC_AD_TYPE_BANNER
import gatewayprotocol.v1.DiagnosticEventRequestOuterClass.DiagnosticAdType.DIAGNOSTIC_AD_TYPE_FULLSCREEN
import com.unity3d.services.core.misc.Utilities.wrapCustomerListener
import gatewayprotocol.v1.HeaderBiddingAdMarkupOuterClass.HeaderBiddingAdMarkup
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.withContext
import kotlinx.coroutines.withTimeoutOrNull
import java.util.UUID
import kotlin.time.ExperimentalTime
import kotlin.time.TimeMark
import kotlin.time.TimeSource

@OptIn(ExperimentalTime::class)
internal class LegacyLoadUseCase(
    private val dispatcher: CoroutineDispatcher,
    private val load: Load,
    private val sendDiagnosticEvent: SendDiagnosticEvent,
    private val getInitializationState: GetInitializationState,
    private val awaitInitialization: AwaitInitialization,
    private val sessionRepository: SessionRepository,
    private val adRepository: AdRepository,
) {
    private var isHeaderBidding: Boolean = false
    private var isBanner: Boolean = false
    private var listener: IUnityAdsLoadListener? = null
    private var startTime: TimeMark? = null
    private var placement: String? = null
    private var opportunity: ByteString? = null
    private var adMarkup: String? = null
    private lateinit var loadOptions: UnityAdsLoadOptions

    suspend operator fun invoke(
        context: Context,
        placement: String?,
        loadOptions: UnityAdsLoadOptions,
        unityLoadListener: IUnityAdsLoadListener?,
        bannerSize: UnityBannerSize? = null,
    ) {
        DeviceLog.debug("Unity Ads Load Start for placement $placement")

        val timeoutMillis = sessionRepository.nativeConfiguration.adOperations.loadTimeoutMs.toLong()
        val gatewayBannerSize = getBannerSize(bannerSize)

        this.loadOptions = loadOptions
        this.isHeaderBidding = !adMarkup.isNullOrBlank()
        this.isBanner = bannerSize != null
        this.listener = unityLoadListener
        this.placement = placement
        this.adMarkup = getAdMarkup(loadOptions)

        this.startTime = loadStart()

        try {
            val loadResult = withTimeoutOrNull(timeoutMillis) {

                if (placement == null) {
                    return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INVALID_ARGUMENT,
                        message = MSG_PLACEMENT_NULL,
                        reason = REASON_PLACEMENT_NULL,
                    )
                }

                val opportunityId = getOpportunityId(loadOptions)
                    ?: return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INVALID_ARGUMENT,
                        message = MSG_OPPORTUNITY_ID,
                        reason = REASON_OPPORTUNITY_ID,
                    )

                val opportunityIdByteString = UUID.fromString(opportunityId).toByteString()
                opportunity = opportunityIdByteString
                if (adRepository.hasOpportunityId(opportunityIdByteString)) {
                    LoadResult.Failure(
                        error = UnityAdsLoadError.INVALID_ARGUMENT,
                        message = MSG_OPPORTUNITY_ID_USED,
                        reason = REASON_OPPORTUNITY_USED,
                    )
                }

                // For header bidding, the ad markup is passed in the load options and contains the adData
                val headerBiddingAdMarkup = getHeaderBiddingAdMarkup(adMarkup)
                    ?: return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INTERNAL_ERROR,
                        message = MSG_AD_MARKUP_PARSING,
                        reason = REASON_AD_MARKUP_PARSE
                    )

                when(getInitializationState()) {
                    INITIALIZED -> load(context, placement, opportunityIdByteString, headerBiddingAdMarkup, gatewayBannerSize, loadOptions)
                    NOT_INITIALIZED, FAILED -> return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INITIALIZE_FAILED,
                        message = MSG_NOT_INITIALIZED,
                        reason = REASON_NOT_INITIALIZED,
                    )
                    INITIALIZING -> {
                        when (awaitInitialization()) {
                            INITIALIZED -> load(context, placement, opportunityIdByteString, headerBiddingAdMarkup, gatewayBannerSize, loadOptions)
                            FAILED -> LoadResult.Failure(
                                error = UnityAdsLoadError.INITIALIZE_FAILED,
                                message = MSG_NOT_INITIALIZED,
                                reason = REASON_NOT_INITIALIZED,
                            )
                            else -> LoadResult.Failure(
                                error = UnityAdsLoadError.TIMEOUT,
                                message = MSG_TIMEOUT + placement,
                                reason = REASON_TIMEOUT_INITIALIZATION
                            )
                        }
                    }
                }
            } ?: LoadResult.Failure(
                error = UnityAdsLoadError.TIMEOUT,
                message = MSG_TIMEOUT + placement,
                reason = REASON_TIMEOUT
            )
            when (loadResult) {
                is LoadResult.Success -> loadSuccess(loadResult.adObject)
                is LoadResult.Failure -> loadFailure(loadResult)
            }
        } catch (e: Throwable) {
            val loadResult = LoadResult.Failure(
                error = UnityAdsLoadError.INTERNAL_ERROR,
                message = MSG_INTERNAL_ERROR,
                reason = REASON_UNCAUGHT_EXCEPTION,
                reasonDebug = e.getShortenedStackTrace(),
                throwable = e
            )
            loadFailure(loadResult)
        }

    }

    private fun getHeaderBiddingAdMarkup(adMarkup: String?): HeaderBiddingAdMarkup? {
        return if (!adMarkup.isNullOrBlank()) {
            try {
                HeaderBiddingAdMarkup.parseFrom(adMarkup.fromBase64().toByteArray())
            } catch (e: Exception) {
                return null
            }
        } else {
            HeaderBiddingAdMarkup.getDefaultInstance()
        }
    }

    private fun getBannerSize(bannerSize: UnityBannerSize?) = bannerSize?.let {
        gatewayprotocol.v1.bannerSize {
            width = it.width
            height = it.height
        }
    }

    private fun getOpportunityId(unityAdsLoadOptions: UnityAdsLoadOptions): String? {
        return unityAdsLoadOptions.data?.opt(KEY_OBJECT_ID)?.toString()
    }

    private fun getAdMarkup(unityAdsLoadOptions: UnityAdsLoadOptions): String? {
        return unityAdsLoadOptions.data?.opt(KEY_AD_MARKUP)?.toString()
    }

    private fun loadStart(): TimeMark {
        val startTime = TimeSource.Monotonic.markNow()
        sendDiagnosticEvent(
            event = SendDiagnosticEvent.LOAD_STARTED,
            tags = getTags(),
            adObject = getTmpAdObject(),
        )
        return startTime
    }

    private suspend fun loadSuccess(adObject: AdObject) {
        DeviceLog.debug("Unity Ads Load Success for placement: $placement")
        sendDiagnosticEvent(
            event = SendDiagnosticEvent.LOAD_SUCCESS,
            value = startTime?.elapsedMillis(),
            tags = getTags(),
            adObject = adObject,
        )
        withContext(dispatcher) {
            wrapCustomerListener { listener?.onUnityAdsAdLoaded(placement) }
        }
    }

    private suspend fun loadFailure(
        loadResult: LoadResult.Failure
    ) {
        DeviceLog.debug("Unity Ads Load Failure for placement: $placement reason: ${loadResult.error} :: ${loadResult.message}")
        sendDiagnosticEvent(
            event = SendDiagnosticEvent.LOAD_FAILURE,
            value = startTime?.elapsedMillis(),
            tags = getTags(loadResult.reason, loadResult.reasonDebug),
            adObject = getTmpAdObject(),
        )
        withContext(dispatcher) {
            wrapCustomerListener { listener?.onUnityAdsFailedToLoad(placement, loadResult.error, loadResult.message) }
        }
    }

    private fun getTags(
        reason: String? = null,
        reasonDebug: String? = null
    ): Map<String, String> {
        val tags = mutableMapOf(
            STATE to getInitializationState().toString(),
            OPERATION to OperationType.LOAD.toString(),
        )
        if (!reason.isNullOrEmpty()) tags[REASON] = reason
        if (!reasonDebug.isNullOrEmpty()) tags[REASON_DEBUG] = reasonDebug
        return tags
    }

    private fun getAdType(): DiagnosticAdType {
        return if (isBanner) DIAGNOSTIC_AD_TYPE_BANNER else DIAGNOSTIC_AD_TYPE_FULLSCREEN
    }

    private fun getTmpAdObject(): AdObject {
        val tmpOpportunityId = getOpportunityId(loadOptions)
        val tmpOpportunityByteString = if (tmpOpportunityId != null) {
            UUID.fromString(tmpOpportunityId).toByteString()
        } else {
            ByteString.EMPTY
        }
        return AdObject(
            opportunityId = tmpOpportunityByteString,
            placementId = placement ?: "",
            trackingToken = ByteString.EMPTY,
            adPlayer = null,
            loadOptions = loadOptions,
            isHeaderBidding = isHeaderBidding,
            adType = getAdType(),
        )
    }

    companion object {
        const val KEY_OBJECT_ID = "objectId"
        const val KEY_AD_MARKUP = "adMarkup"
    }
}

==> unity-ads-4.14.0-sources.jar: com/unity3d/ads/core/domain/LegacyLoadUseCase.kt <==
package com.unity3d.ads.core.domain

import android.content.Context
import com.google.protobuf.ByteString
import com.unity3d.ads.IUnityAdsLoadListener
import com.unity3d.ads.UnityAds.UnityAdsLoadError
import com.unity3d.ads.UnityAdsLoadOptions
import com.unity3d.ads.core.data.model.AdObject
import com.unity3d.ads.core.data.model.LoadResult
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_AD_MARKUP_PARSING
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_NOT_INITIALIZED
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_OPPORTUNITY_ID
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_OPPORTUNITY_ID_USED
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_PLACEMENT_NULL
import com.unity3d.ads.core.data.model.LoadResult.Companion.MSG_TIMEOUT
import com.unity3d.ads.core.data.model.InitializationState.INITIALIZED
import com.unity3d.ads.core.data.model.InitializationState.NOT_INITIALIZED
import com.unity3d.ads.core.data.model.InitializationState.FAILED
import com.unity3d.ads.core.data.model.InitializationState.INITIALIZING
import com.unity3d.ads.core.data.model.OperationType
import com.unity3d.ads.core.data.repository.AdRepository
import com.unity3d.ads.core.data.repository.SessionRepository
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.OPERATION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_AD_MARKUP_PARSE
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_DEBUG
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_NOT_INITIALIZED
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_OPPORTUNITY_ID
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_OPPORTUNITY_USED
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_PLACEMENT_NULL
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_TIMEOUT
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_TIMEOUT_INITIALIZATION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.REASON_UNCAUGHT_EXCEPTION
import com.unity3d.ads.core.domain.SendDiagnosticEvent.Companion.STATE
import com.unity3d.ads.core.extensions.*
import com.unity3d.ads.core.extensions.toByteString
import com.unity3d.services.UnityAdsConstants.Messages.MSG_INTERNAL_ERROR
import com.unity3d.services.banners.UnityBannerSize
import com.unity3d.services.core.log.DeviceLog
import gatewayprotocol.v1.DiagnosticEventRequestOuterClass.DiagnosticAdType
import gatewayprotocol.v1.DiagnosticEventRequestOuterClass.DiagnosticAdType.DIAGNOSTIC_AD_TYPE_BANNER
import gatewayprotocol.v1.DiagnosticEventRequestOuterClass.DiagnosticAdType.DIAGNOSTIC_AD_TYPE_FULLSCREEN
import com.unity3d.services.core.misc.Utilities.wrapCustomerListener
import gatewayprotocol.v1.HeaderBiddingAdMarkupOuterClass.HeaderBiddingAdMarkup
import kotlinx.coroutines.CoroutineDispatcher
import kotlinx.coroutines.withContext
import kotlinx.coroutines.withTimeoutOrNull
import java.util.UUID
import kotlin.time.ExperimentalTime
import kotlin.time.TimeMark
import kotlin.time.TimeSource

@OptIn(ExperimentalTime::class)
internal class LegacyLoadUseCase(
    private val dispatcher: CoroutineDispatcher,
    private val load: Load,
    private val sendDiagnosticEvent: SendDiagnosticEvent,
    private val getInitializationState: GetInitializationState,
    private val awaitInitialization: AwaitInitialization,
    private val sessionRepository: SessionRepository,
    private val adRepository: AdRepository,
) {
    private var isHeaderBidding: Boolean = false
    private var isBanner: Boolean = false
    private var listener: IUnityAdsLoadListener? = null
    private var startTime: TimeMark? = null
    private var placement: String? = null
    private var opportunity: ByteString? = null
    private var adMarkup: String? = null
    private lateinit var loadOptions: UnityAdsLoadOptions

    suspend operator fun invoke(
        context: Context,
        placement: String?,
        loadOptions: UnityAdsLoadOptions,
        unityLoadListener: IUnityAdsLoadListener?,
        bannerSize: UnityBannerSize? = null,
    ) {
        DeviceLog.debug("Unity Ads Load Start for placement $placement")

        val timeoutMillis = sessionRepository.nativeConfiguration.adOperations.loadTimeoutMs.toLong()
        val gatewayBannerSize = getBannerSize(bannerSize)

        this.loadOptions = loadOptions
        this.adMarkup = getAdMarkup(loadOptions)

        this.isHeaderBidding = !adMarkup.isNullOrBlank()
        this.isBanner = bannerSize != null
        this.listener = unityLoadListener
        this.placement = placement

        this.startTime = loadStart()

        try {
            val loadResult = withTimeoutOrNull(timeoutMillis) {

                if (placement == null) {
                    return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INVALID_ARGUMENT,
                        message = MSG_PLACEMENT_NULL,
                        reason = REASON_PLACEMENT_NULL,
                    )
                }

                val opportunityId = getOpportunityId(loadOptions)
                    ?: return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INVALID_ARGUMENT,
                        message = MSG_OPPORTUNITY_ID,
                        reason = REASON_OPPORTUNITY_ID,
                    )

                val opportunityIdByteString = UUID.fromString(opportunityId).toByteString()
                opportunity = opportunityIdByteString
                if (adRepository.hasOpportunityId(opportunityIdByteString)) {
                    LoadResult.Failure(
                        error = UnityAdsLoadError.INVALID_ARGUMENT,
                        message = MSG_OPPORTUNITY_ID_USED,
                        reason = REASON_OPPORTUNITY_USED,
                    )
                }

                // For header bidding, the ad markup is passed in the load options and contains the adData
                val headerBiddingAdMarkup = getHeaderBiddingAdMarkup(adMarkup)
                    ?: return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INTERNAL_ERROR,
                        message = MSG_AD_MARKUP_PARSING,
                        reason = REASON_AD_MARKUP_PARSE
                    )

                when(getInitializationState()) {
                    INITIALIZED -> load(context, placement, opportunityIdByteString, headerBiddingAdMarkup, gatewayBannerSize, loadOptions)
                    NOT_INITIALIZED, FAILED -> return@withTimeoutOrNull LoadResult.Failure(
                        error = UnityAdsLoadError.INITIALIZE_FAILED,
                        message = MSG_NOT_INITIALIZED,
                        reason = REASON_NOT_INITIALIZED,
                    )
                    INITIALIZING -> {
                        when (awaitInitialization()) {
                            INITIALIZED -> load(context, placement, opportunityIdByteString, headerBiddingAdMarkup, gatewayBannerSize, loadOptions)
                            FAILED -> LoadResult.Failure(
                                error = UnityAdsLoadError.INITIALIZE_FAILED,
                                message = MSG_NOT_INITIALIZED,
                                reason = REASON_NOT_INITIALIZED,
                            )
                            else -> LoadResult.Failure(
                                error = UnityAdsLoadError.TIMEOUT,
                                message = MSG_TIMEOUT + placement,
                                reason = REASON_TIMEOUT_INITIALIZATION
                            )
                        }
                    }
                }
            } ?: LoadResult.Failure(
                error = UnityAdsLoadError.TIMEOUT,
                message = MSG_TIMEOUT + placement,
                reason = REASON_TIMEOUT
            )
            when (loadResult) {
                is LoadResult.Success -> loadSuccess(loadResult.adObject)
                is LoadResult.Failure -> loadFailure(loadResult)
            }
        } catch (e: Throwable) {
            val loadResult = LoadResult.Failure(
                error = UnityAdsLoadError.INTERNAL_ERROR,
                message = MSG_INTERNAL_ERROR,
                reason = REASON_UNCAUGHT_EXCEPTION,
                reasonDebug = e.getShortenedStackTrace(),
                throwable = e
            )
            loadFailure(loadResult)
        }

    }

    private fun getHeaderBiddingAdMarkup(adMarkup: String?): HeaderBiddingAdMarkup? {
        return if (!adMarkup.isNullOrBlank()) {
            try {
                HeaderBiddingAdMarkup.parseFrom(adMarkup.fromBase64().toByteArray())
            } catch (e: Exception) {
                return null
            }
        } else {
            HeaderBiddingAdMarkup.getDefaultInstance()
        }
    }

    private fun getBannerSize(bannerSize: UnityBannerSize?) = bannerSize?.let {
        gatewayprotocol.v1.bannerSize {
            width = it.width
            height = it.height
        }
    }

    private fun getOpportunityId(unityAdsLoadOptions: UnityAdsLoadOptions): String? {
        return unityAdsLoadOptions.data?.opt(KEY_OBJECT_ID)?.toString()
    }

    private fun getAdMarkup(unityAdsLoadOptions: UnityAdsLoadOptions): String? {
        return unityAdsLoadOptions.data?.opt(KEY_AD_MARKUP)?.toString()
    }

    private fun loadStart(): TimeMark {
        val startTime = TimeSource.Monotonic.markNow()
        sendDiagnosticEvent(
            event = SendDiagnosticEvent.LOAD_STARTED,
            tags = getTags(),
            adObject = getTmpAdObject(),
        )
        return startTime
    }

    private suspend fun loadSuccess(adObject: AdObject) {
        DeviceLog.debug("Unity Ads Load Success for placement: $placement")
        sendDiagnosticEvent(
            event = SendDiagnosticEvent.LOAD_SUCCESS,
            value = startTime?.elapsedMillis(),
            tags = getTags(),
            adObject = adObject,
        )
        withContext(dispatcher) {
            wrapCustomerListener { listener?.onUnityAdsAdLoaded(placement) }
        }
    }

    private suspend fun loadFailure(
        loadResult: LoadResult.Failure
    ) {
        DeviceLog.debug("Unity Ads Load Failure for placement: $placement reason: ${loadResult.error} :: ${loadResult.message}")
        sendDiagnosticEvent(
            event = SendDiagnosticEvent.LOAD_FAILURE,
            value = startTime?.elapsedMillis(),
            tags = getTags(loadResult.reason, loadResult.reasonDebug),
            adObject = getTmpAdObject(loadResult.isScarAd),
        )
        withContext(dispatcher) {
            wrapCustomerListener { listener?.onUnityAdsFailedToLoad(placement, loadResult.error, loadResult.message) }
        }
    }

    private fun getTags(
        reason: String? = null,
        reasonDebug: String? = null
    ): Map<String, String> {
        val tags = mutableMapOf(
            STATE to getInitializationState().toString(),
            OPERATION to OperationType.LOAD.toString(),
        )
        if (!reason.isNullOrEmpty()) tags[REASON] = reason
        if (!reasonDebug.isNullOrEmpty()) tags[REASON_DEBUG] = reasonDebug
        return tags
    }

    private fun getAdType(): DiagnosticAdType {
        return if (isBanner) DIAGNOSTIC_AD_TYPE_BANNER else DIAGNOSTIC_AD_TYPE_FULLSCREEN
    }

    private fun getTmpAdObject(isScarAd: Boolean = false): AdObject {
        val tmpOpportunityId = getOpportunityId(loadOptions)
        val tmpOpportunityByteString = if (tmpOpportunityId != null) {
            UUID.fromString(tmpOpportunityId).toByteString()
        } else {
            ByteString.EMPTY
        }
        return AdObject(
            opportunityId = tmpOpportunityByteString,
            placementId = placement ?: "",
            trackingToken = ByteString.EMPTY,
            adPlayer = null,
            loadOptions = loadOptions,
            isHeaderBidding = isHeaderBidding,
            adType = getAdType(),
            isScarAd = isScarAd,
        )
    }

    companion object {
        const val KEY_OBJECT_ID = "objectId"
        const val KEY_AD_MARKUP = "adMarkup"
    }
}
